@@ -1,3 +1,4 @@
+
 package com.openexchange.spamhandler.spamexperts.servlets;
 
 /*
@@ -53,16 +54,19 @@ import java.io.IOException;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
+import org.apache.http.impl.client.CloseableHttpClient;
 import org.json.JSONException;
 import org.json.JSONObject;
 import com.openexchange.ajax.DataServlet;
 import com.openexchange.ajax.container.Response;
 import com.openexchange.exception.OXException;
-import com.openexchange.groupware.contexts.Context;
-import com.openexchange.groupware.contexts.impl.ContextStorage;
+import com.openexchange.rest.client.httpclient.HttpClients;
+import com.openexchange.rest.client.httpclient.HttpClients.ClientConfig;
 import com.openexchange.session.Session;
+import com.openexchange.spamhandler.spamexperts.management.SpamExpertsConfig;
 import com.openexchange.tools.servlet.OXJSONExceptionCodes;
 import com.openexchange.tools.session.ServerSession;
+import com.openexchange.version.Version;
 
 /**
  *
@@ -77,58 +81,78 @@ import com.openexchange.tools.session.ServerSession;
  */
 public final class SpamExpertsServlet extends DataServlet {
 
-	/**
-	 *
-	 */
-	private static final long serialVersionUID = -8914926421736440078L;
-	private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(SpamExpertsServlet.class);
+    private static final long serialVersionUID = -8914926421736440078L;
 
-	public SpamExpertsServlet() {
-		super();
-	}
+    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(SpamExpertsServlet.class);
 
-	@Override
-	protected boolean hasModulePermission(final ServerSession session) {
-		return true;
-	}
+    private final SpamExpertsConfig config;
+    private final CloseableHttpClient httpClient;
 
-	@Override
-    protected void doGet(final HttpServletRequest req,
-			final HttpServletResponse resp) throws ServletException,
-			IOException {
+    /**
+     * Initializes a new {@link SpamExpertsServlet}.
+     *
+     * @param config The configuration to use
+     */
+    public SpamExpertsServlet(SpamExpertsConfig config) {
+        super();
+        this.config = config;
 
-		final Response response = new Response();
-		final Session session = getSessionObject(req);
+        String versionString = Version.getInstance().optVersionString();
+        if (null == versionString) {
+            versionString = "<unknown version>";
+        }
+        httpClient = HttpClients.getHttpClient(ClientConfig.newInstance()
+            .setUserAgent("OX Spam Experts Client v" + versionString)
+            .setMaxTotalConnections(32)
+            .setMaxConnectionsPerRoute(32)
+            .setConnectionTimeout(5000)
+            .setSocketReadTimeout(30000)
+        );
+    }
 
-		try {
+    @Override
+    protected boolean hasModulePermission(final ServerSession session) {
+        return true;
+    }
 
-			final String action = parseMandatoryStringParameter(req,PARAMETER_ACTION);
-			JSONObject jsonObj;
+    @Override
+    protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
+        Session session = getSessionObject(req);
 
-			try {
-				jsonObj = convertParameter2JSONObject(req);
-			} catch (final JSONException e) {
-				LOG.error("", e);
-				response.setException(OXJSONExceptionCodes.JSON_BUILD_ERROR.create(e));
-				writeResponse(response, resp, session);
-				return;
-			}
-			final Context ctx = ContextStorage.getInstance().getContext(session);
-			final SpamExpertsServletRequest proRequest = new SpamExpertsServletRequest(session, ctx);
-			final Object responseObj = proRequest.action(action, jsonObj);
-			response.setData(responseObj);
+        Response response = new Response();
+        try {
+            String action = parseMandatoryStringParameter(req, PARAMETER_ACTION);
 
-		} catch (final OXException e) {
-			LOG.error("", e);
-			response.setException(e);
-		} catch (final JSONException e) {
-			final OXException oje = OXJSONExceptionCodes.JSON_WRITE_ERROR.create(e);
-			LOG.error("", oje);
-			response.setException(oje);
-		}
+            JSONObject jsonObj;
+            try {
+                jsonObj = convertParameter2JSONObject(req);
+            } catch (final JSONException e) {
+                LOG.error("", e);
+                response.setException(OXJSONExceptionCodes.JSON_BUILD_ERROR.create(e));
+                writeResponse(response, resp, session);
+                return;
+            }
 
-		writeResponse(response, resp, session);
+            SpamExpertsServletRequest proRequest = new SpamExpertsServletRequest(session, config, httpClient);
+            Object responseObj = proRequest.action(action, jsonObj);
+            response.setData(responseObj);
+        } catch (OXException e) {
+            LOG.error("", e);
+            response.setException(e);
+        }
 
-	}
+        writeResponse(response, resp, session);
+    }
+
+    /**
+     * Shuts-down this servlet instance.
+     */
+    public void shutDown() {
+        try {
+            httpClient.close();
+        } catch (Exception e) {
+            // Ignore
+        }
+    }
 
 }
